@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-from __future__ import print_function
 import setuptools  # noqa
 from setuptools import Extension
 
@@ -49,8 +47,11 @@ class NumpyExtension(Extension):
         from os.path import join
         return join(pathname, "core", "include")
 
+    def get_additional_include_dirs(self):
+        return [self.get_numpy_incpath()]
+
     def get_include_dirs(self):
-        return self._include_dirs + [self.get_numpy_incpath()]
+        return self._include_dirs + self.get_additional_include_dirs()
 
     def set_include_dirs(self, value):
         self._include_dirs = value
@@ -66,12 +67,9 @@ class PyUblasExtension(NumpyExtension):
         from pkg_resources import Requirement, resource_filename
         return resource_filename(Requirement.parse(name), "%s/include" % name)
 
-    @property
-    def include_dirs(self):
-        return self._include_dirs + [
-                self.get_numpy_incpath(),
-                self.get_module_include_path("pyublas"),
-                ]
+    def get_additional_include_dirs(self):
+        return (NumpyExtension.get_additional_include_dirs(self)
+                + [self.get_module_include_path("pyublas")])
 
 
 class HedgeExtension(PyUblasExtension):
@@ -222,7 +220,7 @@ def expand_value(v, options):
 
 def expand_options(options):
     return dict(
-            (k, expand_value(v, options)) for k, v in list(options.items()))
+            (k, expand_value(v, options)) for k, v in options.items())
 
 
 class ConfigSchema:
@@ -262,7 +260,7 @@ class ConfigSchema:
 
         exec(compile(contents, filename, "exec"), filevars)
 
-        for key, value in list(filevars.items()):
+        for key, value in filevars.items():
             if key in self.optdict:
                 result[key] = value
 
@@ -280,11 +278,11 @@ class ConfigSchema:
         if "__builtins__" in filevars:
             del filevars["__builtins__"]
 
-        for key, value in list(config.items()):
+        for key, value in config.items():
             if value is not None:
                 filevars[key] = value
 
-        keys = list(filevars.keys())
+        keys = filevars.keys()
         keys.sort()
 
         outf = open(filename, "w")
@@ -339,7 +337,7 @@ class ConfigSchema:
             filevars = {}
             exec(compile(open(cfile, "r").read(), cfile, "exec"), filevars)
 
-            for key, value in list(filevars.items()):
+            for key, value in filevars.items():
                 if key in self.optdict:
                     result[key] = value
                 elif key == "__builtins__":
@@ -494,7 +492,7 @@ class BoostLibraries(Libraries):
         Libraries.__init__(self, "BOOST_%s" % lib_base_name.upper(),
                 [default_lib_name],
                 help="Library names for Boost C++ %s library (without lib or .so)"
-                    % humanize(lib_base_name))
+                % humanize(lib_base_name))
 
 
 def set_up_shipped_boost_if_requested(project_name, conf, source_path=None,
@@ -518,7 +516,7 @@ def set_up_shipped_boost_if_requested(project_name, conf, source_path=None,
             print(DASH_SEPARATOR)
             print("The shipped Boost library was not found, but "
                     "USE_SHIPPED_BOOST is True.")
-            print(("(The files should be under %s/.)" % source_path))
+            print("(The files should be under %s/.)" % source_path)
             print(DASH_SEPARATOR)
             print("If you got this package from git, you probably want to do")
             print("")
@@ -618,7 +616,7 @@ def configure_frontend():
         print("*** I have detected that you have already run configure.")
         print("*** I'm taking the configured values as defaults for this")
         print("*** configure run. If you don't want this, delete the file")
-        print(("*** %s." % schema.get_conf_file()))
+        print("*** %s." % schema.get_conf_file())
         print("************************************************************")
 
     import sys
@@ -705,6 +703,7 @@ def substitute(substitutions, fname):
 def _run_git_command(cmd):
     git_error = None
     from subprocess import Popen, PIPE
+    stdout = None
     try:
         popen = Popen(["git"] + cmd, stdout=PIPE)
         stdout, stderr = popen.communicate()
@@ -724,8 +723,11 @@ def _run_git_command(cmd):
         print(git_error)
         print("Hit Ctrl-C now if you'd like to think about the situation.")
         print(DASH_SEPARATOR)
-        count_down_delay(delay=5)
-    return stdout.decode("utf-8"), git_error
+        count_down_delay(delay=0)
+    if stdout:
+        return stdout.decode("utf-8"), git_error
+    else:
+        return '', "(subprocess call to git did not succeed)"
 
 
 def check_git_submodules():
@@ -785,7 +787,7 @@ def check_git_submodules():
             print("")
             print("These issues were found:")
             for w in pkg_warnings:
-                print(("  %s" % w))
+                print("  %s" % w)
             print("")
             print("I will try to initialize the submodules for you "
                     "after a short wait.")
