@@ -13,6 +13,7 @@ from pycuda.compyte.array import (
 from pycuda.characterize import has_double_support
 import six
 from six.moves import range, zip, reduce
+import numbers
 
 
 def _get_common_dtype(obj1, obj2):
@@ -167,9 +168,15 @@ class GPUArray(object):
             for dim in shape:
                 s *= dim
         except TypeError:
-            assert isinstance(shape, (int, int, np.integer))
+            # handle dim-0 ndarrays:
+            if isinstance(shape, np.ndarray):
+                shape = np.asscalar(shape)
+            assert isinstance(shape, numbers.Integral)
             s = shape
             shape = (shape,)
+        else:
+            # handle shapes that are ndarrays
+            shape = tuple(shape)
 
         if isinstance(s, np.integer):
             # bombs if s is a Python integer
@@ -236,7 +243,7 @@ class GPUArray(object):
             _memcpy_discontig(self, ary, async=async, stream=stream)
 
     def set_async(self, ary, stream=None):
-        return self.set(ary, async=True, stream=None)
+        return self.set(ary, async=True, stream=stream)
 
     def get(self, ary=None, pagelocked=False, async=False, stream=None):
         if ary is None:
@@ -742,6 +749,23 @@ class GPUArray(object):
                 base=self,
                 gpudata=int(self.gpudata))
 
+    def squeeze(self):
+        """
+        Returns a view of the array with dimensions of
+        length 1 removed.
+        """
+        new_shape = tuple([dim for dim in self.shape if dim > 1])
+        new_strides = tuple([self.strides[i]
+            for i, dim in enumerate(self.shape) if dim > 1])
+
+        return GPUArray(
+            shape=new_shape,
+            dtype=self.dtype,
+            allocator=self.allocator,
+            strides=new_strides,
+            base=self,
+            gpudata=int(self.gpudata))
+
     def transpose(self, axes=None):
         """Permute the dimensions of an array.
 
@@ -1074,9 +1098,12 @@ def arange(*args, **kwargs):
 
 # }}}
 
+
 def _compact_strides(a):
     # Compute strides to have same order as self, but packed
-    info = sorted((a.strides[axis], a.shape[axis], axis) for axis in xrange(len(a.shape)))
+    info = sorted(
+            (a.strides[axis], a.shape[axis], axis)
+            for axis in xrange(len(a.shape)))
 
     strides = [None]*len(a.shape)
     stride = a.dtype.itemsize
@@ -1085,6 +1112,7 @@ def _compact_strides(a):
         stride *= dim
     return strides
 
+
 def _memcpy_discontig(dst, src, async=False, stream=None):
     """Copy the contents of src into dst.
 
@@ -1113,7 +1141,9 @@ def _memcpy_discontig(dst, src, async=False, stream=None):
     else:
         # put src in Fortran order (which should put dst in Fortran order too)
         # and remove singleton axes
-        src_info = sorted((src.strides[axis], axis) for axis in xrange(len(src.shape)) if src.shape[axis] > 1)
+        src_info = sorted(
+                (src.strides[axis], axis)
+                for axis in xrange(len(src.shape)) if src.shape[axis] > 1)
         axes = [axis for _, axis in src_info]
         shape = [src.shape[axis] for axis in axes]
         src_strides = [src.strides[axis] for axis in axes]
@@ -1191,7 +1221,7 @@ def _memcpy_discontig(dst, src, async=False, stream=None):
 
     if len(shape) == 2:
         if async:
-            copy(stream=stream)
+            copy(stream)
         else:
             copy(aligned=True)
 
@@ -1206,7 +1236,7 @@ def _memcpy_discontig(dst, src, async=False, stream=None):
 
         copy.depth = shape[2]
         if async:
-            copy(stream=stream)
+            copy(stream)
         else:
             copy()
 
