@@ -49,6 +49,12 @@ class TestGPUArray:
         gpuarray.empty(np.int32(17), np.float32)
         gpuarray.empty((np.int32(17), np.int32(17)), np.float32)
 
+    @mark_cuda_test
+    def test_ndarray_shape(self):
+        gpuarray.empty(np.array(3), np.float32)
+        gpuarray.empty(np.array([3]), np.float32)
+        gpuarray.empty(np.array([2, 3]), np.float32)
+
     @mark_cuda_test
     def test_abs(self):
         a = -gpuarray.arange(111, dtype=np.float32)
@@ -745,7 +751,7 @@ class TestGPUArray:
         a_gpu = gpuarray.GPUArray(100, dtype=gpuarray.vec.float3)
         a_gpu.fill(gpuarray.vec.make_float3(0.0, 0.0, 0.0))
         a = a_gpu.get()
-        assert a.dtype is gpuarray.vec.float3
+        assert a.dtype == gpuarray.vec.float3
 
     @mark_cuda_test
     def test_create_complex_zeros(self):
@@ -790,6 +796,38 @@ class TestGPUArray:
         view = a_gpu.view(np.int16)
         assert view.shape == (8, 32) and view.dtype == np.int16
 
+    @mark_cuda_test
+    def test_squeeze(self):
+        shape = (40, 2, 5, 100)
+        a_cpu = np.random.random(size=shape)
+        a_gpu = gpuarray.to_gpu(a_cpu)
+
+        # Slice with length 1 on dimensions 0 and 1
+        a_gpu_slice = a_gpu[0:1,1:2,:,:]
+        assert a_gpu_slice.shape == (1,1,shape[2],shape[3])
+        assert a_gpu_slice.flags.c_contiguous is False 
+
+        # Squeeze it and obtain contiguity
+        a_gpu_squeezed_slice = a_gpu[0:1,1:2,:,:].squeeze()
+        assert a_gpu_squeezed_slice.shape == (shape[2],shape[3])
+        assert a_gpu_squeezed_slice.flags.c_contiguous is True 
+
+        # Check that we get the original values out
+        assert np.all(a_gpu_slice.get().ravel() == a_gpu_squeezed_slice.get().ravel())
+
+        # Slice with length 1 on dimensions 2
+        a_gpu_slice = a_gpu[:,:,2:3,:]
+        assert a_gpu_slice.shape == (shape[0],shape[1],1,shape[3])
+        assert a_gpu_slice.flags.c_contiguous is False
+
+        # Squeeze it, but no contiguity here
+        a_gpu_squeezed_slice = a_gpu[:,:,2:3,:].squeeze()
+        assert a_gpu_squeezed_slice.shape == (shape[0],shape[1],shape[3])
+        assert a_gpu_squeezed_slice.flags.c_contiguous is False
+
+        # Check that we get the original values out
+        assert np.all(a_gpu_slice.get().ravel() == a_gpu_squeezed_slice.get().ravel())
+
     @mark_cuda_test
     def test_struct_reduce(self):
         preamble = """
